<?php
  require_once ('functions.php');
?>

/*
  Functions:

    event-driven functions:
      - onLoadBody ()                       when html-body is loaded
      - onChangeTabCalculation ()           when activating the calculation bar
      - onChangeCalculationItem ()          when changing the item to be calculated in sink-rate-dialog
      - onChangeItem ()                     when changing editable items in sink-rate-dialog
      - onChangeUnits ()                    when changing editable units in ?-dialog
      - onChangeProperties ()               when changing properties in ?-dialog
      - onChangeShowAircraftInfos ()        when selecting an other aircraft to show its infos

    internal functions:
      - setObjectVariables ()               set the calculation input-object-variables
      - setCalculationDefault ()            determine option-group-field that should be calculated
      - setConnotationHeadline ()           display connotation h2 as sub navigation
      - setConnotationHeadlineLinks ()      set connotation headline links (only called, when they have been conversted into html links)
      - setFocusToCalculationElement ()     set focus to most edited field
      - clearCssClassOfInputFields ()       removes css-class from all input fields except the one to be calculated
      - clearCssClassOfInputField ()        removes css-class from the given input field
      - calculateDefaultItem ()             calculate default item
      - calculateItem ()                    calculate given item
      - convertTimeToHMM ()                 converts numeric value to time in h:mm-format

    calculation functions:
      - calculateDistance ()                calculate distance in nm
      - calculateHeight ()                  calculate height in hundreds of feet
      - calculateSpeed ()                   calculate speed in kts
      - calculateRate ()                    calculate rate in ft / min
      - calculateDegrees ()                 calculate slope in degrees
      - calculateTime ()                    calculate time in h:mm
*/

// CONSTANTS

const NUMBER_OF_MINUTES_IN_HOUR        = 60;
const NAUTIC_MILE_IN_FEET              = 6076.12;
const METER_IN_FEET                    = 3.28;
const METER_PER_SECOND_IN_KNOTS        = 1.94;
const KILOMETER_IN_KNOTS               = 0.54;
const KILOMETER_IN_MILES_PER_HOUR      = 0.62;
const MILES_PER_HOUR_IN_KNOTS          = 0.87;

// VARIABLES

// html-input-field-objects
var objHorizontalDistanceInNauticMiles;
var objVerticalDistanceInFeet;
var objGroundSpeedInKts;
var objRateInFtPerMinute;
var objRateInDegrees;
var objAmountOfTime;


// EVENT-DRIVEN FUNCTIONS


function onLoadBody ()
{
  setCalculationDefault ();
  setObjectVariables ();
  setConnotationHeadline ();
}


function onChangeTabCalculation ()
{
  setFocusToCalculationElement ();
  calculateDefaultItem ();
}


function onChangeCalculationItem (item)
{
  var currentCalculationItem = item.value;

  clearCssClassOfInputFields (currentCalculationItem);

  document.getElementById(currentCalculationItem).className = "disabled";
  document.getElementById(currentCalculationItem).disabled = true;
}


function onChangeItem (item)
{
  calculateItem (document.calculation_form.calculation_group.value);
}


function onChangeUnits (item)
{
  var val = 0;
  var objUnits = document.getElementById("units");
  var objInputLeft = document.getElementById("input_left");
  var objInputRight = document.getElementById("input_right");
  var objLabelLeft = document.getElementById("label_input_left");
  var objLabelRight = document.getElementById("label_input_right");

  switch (objUnits.value)
  {
    case 'm_ft':
      if (item.id == "input_left")
      {
        val = item.value * METER_IN_FEET;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / METER_IN_FEET;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "m";
        objInputLeft.value = 0;
        objInputLeft.step = 100;
        objLabelRight.innerHTML = "ft";
        objInputRight.value = 0;
        objInputRight.step = 500;
      }
      break;
    case 'm/s_kts':
      if (item.id == "input_left")
      {
        val = item.value * METER_PER_SECOND_IN_KNOTS;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / METER_PER_SECOND_IN_KNOTS;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "m/s";
        objInputLeft.value = 0;
        objInputLeft.step = 1;
        objLabelRight.innerHTML = "kts";
        objInputRight.value = 0;
        objInputRight.step = 1;
      }
      break;
    case 'kmh_kts':
      if (item.id == "input_left")
      {
        val = item.value * KILOMETER_IN_KNOTS;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / KILOMETER_IN_KNOTS;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "km/h";
        objInputLeft.value = 0;
        objInputLeft.step = 10;
        objLabelRight.innerHTML = "kts";
        objInputRight.value = 0;
        objInputRight.step = 10;
      }
      break;
    case 'kmh_mph':
      if (item.id == "input_left")
      {
        val = item.value * KILOMETER_IN_MILES_PER_HOUR;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / KILOMETER_IN_MILES_PER_HOUR;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "km/h";
        objInputLeft.value = 0;
        objInputLeft.step = 10;
        objLabelRight.innerHTML = "mph";
        objInputRight.value = 0;
        objInputRight.step = 10;
      }
      break;
    case 'mph_kts':
      if (item.id == "input_left")
      {
        val = item.value * MILES_PER_HOUR_IN_KNOTS;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / MILES_PER_HOUR_IN_KNOTS;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "mph";
        objInputLeft.value = 0;
        objInputLeft.step = 10;
        objLabelRight.innerHTML = "kts";
        objInputRight.value = 0;
        objInputRight.step = 10;
      }
      break;
    case 'km_nm':
      if (item.id == "input_left")
      {
        val = item.value * KILOMETER_IN_KNOTS;
        objInputRight.value = val.toFixed(0);
      }
      else if (item.id == "input_right")
      {
        val = item.value / KILOMETER_IN_KNOTS;
        objInputLeft.value = val.toFixed(0);
      }
      else if (item.id == "units")
      {
        objLabelLeft.innerHTML = "km";
        objInputLeft.value = 0;
        objInputLeft.step = 10;
        objLabelRight.innerHTML = "nm";
        objInputRight.value = 0;
        objInputRight.step = 10;
      }
      break;
  }
}


function onChangeProperties (item)
{
  var objTableDiv = document.getElementById("property_table");

  switch (item.value)
  {
    case 'default':
      objTableDiv.innerHTML = "";
      break;
    case 'rw_departure':
      objTableDiv.innerHTML = "<?php echo getPropertyTableData ($arrInfoIni, 'departure-runway-length'); ?>";
      break;
    case 'rw_approach':
      objTableDiv.innerHTML = "<?php echo getPropertyTableData ($arrInfoIni, 'approach-runway-length'); ?>";
      break;
    case 'cruise_speed':
      objTableDiv.innerHTML = "<?php echo getPropertyTableData ($arrInfoIni, 'Vc', true); ?>";
      break;
    case 'service_ceiling':
      objTableDiv.innerHTML = "<?php echo getPropertyTableData ($arrInfoIni, 'cruise-max-altitude'); ?>";
      break;
    case 'range':
      objTableDiv.innerHTML = "<?php echo getPropertyTableData ($arrInfoIni, 'cruise-range'); ?>";
      break;
    default:
      objTableDiv.innerHTML = "<p>unknown value '" + item.value + "'.</p>";
      break;
  }
}


function onChangeShowAircraftInfos (item)
{
  window.location.href = "?aircraft=" + item.value;
}


// INTERNAL FUNCTIONS


function setObjectVariables ()
{
  objHorizontalDistanceInNauticMiles   = document.getElementById('distance-in-nm');
  objVerticalDistanceInFeet            = document.getElementById('height-difference-in-ft');
  objGroundSpeedInKts                  = document.getElementById('speed-in-kts');
  objRateInFtPerMinute                 = document.getElementById('rate-in-ft-per-min');
  objRateInDegrees                     = document.getElementById('rate-in-degrees');
  objAmountOfTime                      = document.getElementById('time-in-hmm');
}


function setCalculationDefault ()
{
  calculationDefaultItem = '<?php echo $arrCalculation['calculation-default-item']; ?>';

  // option-group-value of the element that should be calculated by default
  document.calculation_form.calculation_group.value = calculationDefaultItem;

  // set input-field properties via css-class: border and background-color
  document.getElementById(calculationDefaultItem).className = "disabled";
  document.getElementById(calculationDefaultItem).disabled = true;
}


function setConnotationHeadline ()
{
  var s = '';
  var sHeadline = document.getElementById('connotations').innerHTML;
  var objCommonSpecialsPreparings = document.getElementById('common-specials-preparings');
  var objCommonSpecialsBeforeTakeOff = document.getElementById('common-specials-before-take-off');
  var objCommonSpecialsAfterTakeOff = document.getElementById('common-specials-after-take-off');
  var objCommonSpecialsAutopilot = document.getElementById('common-specials-autopilot');
  var objCommonSpecialsCutOff = document.getElementById('common-specials-cut-off');

  // check additional common-specials for transforming the connotation-headline into a html-link
  if (objCommonSpecialsPreparings || objCommonSpecialsBeforeTakeOff || objCommonSpecialsAfterTakeOff || objCommonSpecialsAutopilot || objCommonSpecialsCutOff)
    sHeadline = '<a href="" name="common-specials" onClick="setConnotationHeadlineLinks (this); return false;" style="color:black;">' + sHeadline + '</a>';

  // check additional common-specials for extending the connotations-headline

  if (objCommonSpecialsPreparings)
    s = s + ' / <a href="" name="common-specials-preparings" onClick="setConnotationHeadlineLinks (this); return false;" style="color:grey;">preparings</a>';

  if (objCommonSpecialsBeforeTakeOff)
    s = s + ' / <a href="" name="common-specials-before-take-off" onClick="setConnotationHeadlineLinks (this); return false;" style="color:grey;">&lt; take-off</a>';

  if (objCommonSpecialsAfterTakeOff)
    s = s + ' / <a href="" name="common-specials-after-take-off" onClick="setConnotationHeadlineLinks (this); return false;" style="color:grey;">&gt; take-off</a>';

  if (objCommonSpecialsAutopilot)
    s = s + ' / <a href="" name="common-specials-autopilot" onClick="setConnotationHeadlineLinks (this); return false;" style="color:grey;">autopilot</a>';

  if (objCommonSpecialsCutOff)
    s = s + ' / <a href="" name="common-specials-cut-off" onClick="setConnotationHeadlineLinks (this); return false;" style="color:grey;">cut-off</a>';

  // returning connotations-headline with html links
  if (s)
    document.getElementById('connotations').innerHTML = sHeadline + s;
}


function setConnotationHeadlineLinks (obj)
{
  var clickedLinkText = obj.innerHTML;
  var connotations = document.getElementById('connotations');

  for (var link=connotations.firstChild; link; link=link.nextSibling)
  {
    // loop through the html links inside the extended connotation headline, setting headline links and hiding/displaying according content
    if (link.tagName == 'A')
    {
      if (clickedLinkText == link.innerHTML)
      {
        link.style.color = 'black';
        document.getElementById(link.name).style.display = 'block';
      }
      else
      {
        link.style.color = 'grey';
        document.getElementById(link.name).style.display = 'none';
      }
    }
  }
}


function setFocusToCalculationElement ()
{
  // set focus to the calculation element as given in the ini-file
  document.getElementById('<?php echo $arrCalculation['calculation-default-html-focus-item']; ?>').focus();
}

function clearCssClassOfInputFields (currentCalculationItem)
{
  clearCssClassOfInputField (currentCalculationItem, 'distance-in-nm');
  clearCssClassOfInputField (currentCalculationItem, 'height-difference-in-ft');
  clearCssClassOfInputField (currentCalculationItem, 'speed-in-kts');
  clearCssClassOfInputField (currentCalculationItem, 'rate-in-ft-per-min');
}


function clearCssClassOfInputField (currentCalculationItem, item)
{
  if (currentCalculationItem != item)
  {
    document.getElementById (item).className = '';
    document.getElementById (item).disabled = false;
  }
}


function calculateDefaultItem ()
{
  calculateItem ('<?php echo $arrCalculation['calculation-default-item']; ?>');
}


function calculateItem (itemHtmlId)
{
  switch (itemHtmlId)
  {
    case 'distance-in-nm':
      calculateDistance ();
      break;
    case 'height-difference-in-ft':
      calculateHeight ();
      break;
    case 'speed-in-kts':
      calculateSpeed ();
      break;
    case 'rate-in-ft-per-min':
      calculateRate ();
      break;
    default:
      calculateItem ('distance-in-nm');
  }

  calculateDegrees ();
  calculateTime ();
}


function convertTimeToHMM (num)
{
  num = parseInt (num);

  var hours = Math.floor (num / NUMBER_OF_MINUTES_IN_HOUR);
  var minutes = num % NUMBER_OF_MINUTES_IN_HOUR;

  minutes = minutes.toString ();

  return hours + ":" + minutes.padStart (2, '0');
}


// CALCULATION FUNCTIONS


function calculateDistance ()
{
  var distance = parseInt (objGroundSpeedInKts.value) * parseInt (objVerticalDistanceInFeet.value)  / parseInt (objRateInFtPerMinute.value) / NUMBER_OF_MINUTES_IN_HOUR;

  if (Number.isInteger(distance))
    objHorizontalDistanceInNauticMiles.value = distance;
  else
  {
    distance = distance.toFixed(1);
    objHorizontalDistanceInNauticMiles.value = distance;
  }
}


function calculateHeight ()
{
  var height = parseFloat (objHorizontalDistanceInNauticMiles.value) / parseInt (objGroundSpeedInKts.value) * parseInt (objRateInFtPerMinute.value) * NUMBER_OF_MINUTES_IN_HOUR;
  var heightInHundreds = height / 100;

  heightInHundreds = heightInHundreds.toFixed(0);
  height = heightInHundreds * 100;

  objVerticalDistanceInFeet.value = height;
}


function calculateSpeed ()
{
  var speed = parseFloat (objHorizontalDistanceInNauticMiles.value) / parseInt (objVerticalDistanceInFeet.value) * parseInt (objRateInFtPerMinute.value) * NUMBER_OF_MINUTES_IN_HOUR;

  objGroundSpeedInKts.value = speed.toFixed(0);
}


function calculateRate ()
{
  var rate = parseInt (objGroundSpeedInKts.value) / NUMBER_OF_MINUTES_IN_HOUR / parseFloat (objHorizontalDistanceInNauticMiles.value) * parseInt (objVerticalDistanceInFeet.value);

  objRateInFtPerMinute.value = rate.toFixed(0);
}


function calculateDegrees ()
{
  var horizontalDistance = parseFloat (objHorizontalDistanceInNauticMiles.value) * NAUTIC_MILE_IN_FEET;
  var verticalDistance = parseInt (objVerticalDistanceInFeet.value);

  var gradientInPercent = verticalDistance / horizontalDistance;
  var gradientInDegree  = Math.atan (gradientInPercent) * 180 / Math.PI;

  objRateInDegrees.value = gradientInDegree.toFixed(1);
}


function calculateTime ()
{
  var horizontalDistance = parseFloat (objHorizontalDistanceInNauticMiles.value);
  var groundSpeed = parseInt (objGroundSpeedInKts.value);

  objAmountOfTime.value = convertTimeToHMM (horizontalDistance / groundSpeed * NUMBER_OF_MINUTES_IN_HOUR);
}
