<?php

  /*
    functions:
    - getDataTableRows ()             get content of table rows (tabs: departure, cruise and approach)
    - getAdditionalConnotations ()    get additional content of connotations
    - getAdditionalConnotations2 ()   helper script
    - getCalculationValues ()         insert values into the array of calculation elements
    - checkArrayKey ()                check existance of array keys, setting no-data-value when fail
    - setUndefinedValues ()           set undefined values when working with new aircraft entries
    - getPropertyTableData ()         extract data from $arrInfoIni to show aircraft-property as html table
    - getOtherAircraftInfos ()        list other aircraft names for switching to their data
  */

  // CONSTANTS
  define ('CUSTOM_FLIGHT', getenv('USERPROFILE') . '\AppData\Local\Packages\Microsoft.FlightSimulator_8wekyb3d8bbwe\LocalState\MISSIONS\Custom\CustomFlight\CustomFlight.FLT');
  define ('AIRCRAFT_NAME_SEPARATOR', ' ');
  define ('INFO_INI', 'aircraft.ini');
  define ('INFO_INI_SEPARATOR', '|');
  define ('INFO_INI_NO_DATA', 'no data');
  define ('GET_PARAMETER', 'aircraft');
  define ('CR', "\n");
  define ('TABLE_ROW_INTENT', str_repeat(' ', 10));

  // VARIABLES
  $sAircraftName = '';
  $sMyAircraftName = '';
  $sAircraftFullName = '';
  $arrAircraftName = array();
  $arrCustomFlight = array();
  $arrInfoIni = array();
  $arrCalculation = array();
  $arrCurrentAircraft = array();

  // get aircraft name
  if (isset ($_GET[GET_PARAMETER]))
  {
    // via GET-Parameter
    $sAircraftName = filter_input (INPUT_GET, GET_PARAMETER, FILTER_SANITIZE_STRING);
  }
  else
  {
    // via custom flight file (parsing with sections in raw mode)
    $arrCustomFlight = parse_ini_file (CUSTOM_FLIGHT, true, INI_SCANNER_RAW);
    // read full name
    $sAircraftFullName = $arrCustomFlight['Sim.0']['Sim'];
    // and extract 2-word-name as used here
    $arrAircraftName = explode (AIRCRAFT_NAME_SEPARATOR, $sAircraftFullName);

    // aircraftname in MSFS 2020
    if (count ($arrAircraftName) > 1)
      $sAircraftName = $arrAircraftName[0] . AIRCRAFT_NAME_SEPARATOR . $arrAircraftName[1];
    else
      $sAircraftName = $arrAircraftName[0];
  }

  // parse info-ini with sections in raw mode
  $arrInfoIni = parse_ini_file (INFO_INI, true, INI_SCANNER_RAW);

  // read aircraft-related ini-data
  if (array_key_exists ($sAircraftName, $arrInfoIni))
    $arrCurrentAircraft = $arrInfoIni[$sAircraftName];
  else
  {
    $arrCurrentAircraft[0] = INFO_INI_NO_DATA;
    $arrCurrentAircraft['common-infos'] = INFO_INI_NO_DATA;
    $arrCurrentAircraft['common-specials'] = INFO_INI_NO_DATA;
    $arrCurrentAircraft['calculation-default-distance'] = $arrInfoIni['Defaults']['calculation-default-distance'];
    $arrCurrentAircraft['calculation-default-height'] = $arrInfoIni['Defaults']['calculation-default-height'];
    $arrCurrentAircraft['calculation-default-velocity'] = $arrInfoIni['Defaults']['calculation-default-velocity'];
    $arrCurrentAircraft['calculation-default-rate'] = $arrInfoIni['Defaults']['calculation-default-rate'];
  }

  // set some values to avoid 'undefined index'-messages when working with new aircraft-entries
  setUndefinedValues ($arrCurrentAircraft['common-infos']);
  setUndefinedValues ($arrCurrentAircraft['common-specials']);

  // my aircraftname in aircraft.ini
  $sMyAircraftName = isset ($arrCurrentAircraft['common-name']) ? $arrCurrentAircraft['common-name'] : INFO_INI_NO_DATA;

  // get calculation values
  $arrCalculation = getCalculationValues ($arrInfoIni['Calculation']);

/*
  echo '<pre>';
  print_r ($arrInfoIni);
  print_r ($arrCalculation);
  echo '</pre>';
*/

  // FUNCTIONS


  function getDataTableRows ($arrCurrentAircraft, $sMode)
  {
    $s = CR;
    $arr = array();

    if (array_key_exists (0, $arrCurrentAircraft))
    {
      if ($arrCurrentAircraft[0] == INFO_INI_NO_DATA)
        return CR . TABLE_ROW_INTENT . '<tr><td>'. $arrCurrentAircraft[0] . '</td></tr>' . CR;
    }

    foreach ($arrCurrentAircraft as $key => $val)
    {
      if (strpos ($key, $sMode) === 0)
      {
        $arr = explode (INFO_INI_SEPARATOR, $val);

        // skip output of short-field-info in departure-mode, because it will be printed afterwards in the short-field-mode
        if ($sMode == 'departure' && strpos ($key, 'short-field'))
          continue;

        switch (count ($arr))
        {
          case 2:
            $s .= TABLE_ROW_INTENT . '<tr><td>'. $arr[0] . '</td><td>' . $arr[1] . '</td></tr>' . CR;
            break;
          case 3:
            $s .= TABLE_ROW_INTENT . '<tr><td>'. $arr[0] . '</td><td>' . $arr[1] . '</td><td>' . $arr[2] . '</td></tr>' . CR;
            break;
          default:
            $s .= TABLE_ROW_INTENT . '<tr><td><b>'. $arr[0] . '</b></td><td>' . $arr[1] . '</td><td><b>' . $arr[2] . '</b></td><td>' . $arr[3] . '</td></tr>' . CR;
        }
      }
    }

    return $s;
  }


  function getAdditionalConnotations ($arrCurrentAircraft)
  {
    $s = '';

    $s .= getAdditionalConnotations2 ('common-specials-preparings', $arrCurrentAircraft);
    $s .= getAdditionalConnotations2 ('common-specials-before-take-off', $arrCurrentAircraft);
    $s .= getAdditionalConnotations2 ('common-specials-after-take-off', $arrCurrentAircraft);
    $s .= getAdditionalConnotations2 ('common-specials-autopilot', $arrCurrentAircraft);
    $s .= getAdditionalConnotations2 ('common-specials-cut-off', $arrCurrentAircraft);

    return $s;
  }


  function getAdditionalConnotations2 ($sKey, $arrCurrentAircraft)
  {
    $s = '';

    if (array_key_exists ($sKey, $arrCurrentAircraft))
      $s .= CR . '<div id="' . $sKey . '" style="display:none;">' . $arrCurrentAircraft[$sKey] . '</div>';

    return $s;
  }


  function getCalculationValues ($arrCalculation)
  {
    $arr = array();

    // set captions
    $arr['calculation-default-item'] = checkArrayKey ('calculation-default-item', $arrCalculation);
    $arr['calculation-default-html-focus-item'] = checkArrayKey ('calculation-default-html-focus-item', $arrCalculation);
    $arr['calculation-caption-distance-in-nm'] = checkArrayKey ('calculation-caption-distance-in-nm', $arrCalculation);
    $arr['calculation-caption-height-difference-in-ft'] = checkArrayKey ('calculation-caption-height-difference-in-ft', $arrCalculation);
    $arr['calculation-caption-speed-in-kts'] = checkArrayKey ('calculation-caption-speed-in-kts', $arrCalculation);
    $arr['calculation-caption-rate-in-ft-per-min'] = checkArrayKey ('calculation-caption-rate-in-ft-per-min', $arrCalculation);
    $arr['calculation-caption-rate-in-degrees'] = checkArrayKey ('calculation-caption-rate-in-degrees', $arrCalculation);
    $arr['calculation-caption-time-in-hmm'] = checkArrayKey ('calculation-caption-time-in-hmm', $arrCalculation);

    // set html input values
    $arr['calculation-default-distance'] = checkArrayKey ('calculation-default-distance', $arrCalculation);
    $arr['calculation-default-height'] = checkArrayKey ('calculation-default-height', $arrCalculation);
    $arr['calculation-default-velocity'] = checkArrayKey ('calculation-default-velocity', $arrCalculation);
    $arr['calculation-default-rate'] = checkArrayKey ('calculation-default-rate', $arrCalculation);

    return $arr;
  }


  function checkArrayKey ($key, $arr)
  {
    if (array_key_exists ($key, $arr))
      return $arr[$key];
    else
      return INFO_INI_NO_DATA;
  }


  function setUndefinedValues (&$val)
  {
    if (!isset($val))
      $val = INFO_INI_NO_DATA;
  }


  function getPropertyTableData ($arrII, $sPropertyValue, $bSearchingValues = false)
  {
    $s       = "<table class='r3'>";
    $sVal    = '';
    $sMyName = '';
    $bFound  = false;
    $arr     = array ();
    $arr1    = array ();

    // $arrII (as given as normal parameter) is a copy of $arrInfoIni.
    // Therefor we can delete parts of the array without any harm.

    // clean unnecessary sub-arrays in $arrInfoIni
    unset ($arrII['Calculation']);
    unset ($arrII['Defaults']);

    // clean unnecessary keys in aircraft subarrays
    foreach ($arrII as $key => $val)
    {
      $bFound = false;
      $sMyName = $val['common-name'];

      if ($bSearchingValues == true)
      {
        // search in subarray-values
        foreach ($val as $key1 => $val1)
        {
          if (!(strpos ($val1, $sPropertyValue) === false))
          {
            // read first entry containing the given criteria
            $arr1 = explode ('|', $val1);
            $arr[$sMyName . '|' . $arr1[3]] = $arr1[2];
            $bFound = true;
            break;
          }
        }
      }
      else
      {
        // search in subarray-keys
        foreach ($val as $key1 => $val1)
        {
          if (!(strpos ($key1, $sPropertyValue) === false))
          {
            // read first entry containing the given criteria
            $arr1 = explode ('|', $val1);

            // extract thousand-separators '.' and ',' from values to get clean numeric data to be sorted afterwards
            $sVal = str_replace ('.', '', $arr1[2]);
            $sVal = str_replace (',', '', $sVal);

            $arr[$sMyName . '|' . $arr1[3]] = $sVal;
            $bFound = true;
            break;
          }
        }
      }
      if ($bFound == false)
        $arr[$sMyName . '|' . $arr1[3]] = INFO_INI_NO_DATA;
    }

    // $arr now contains data like: [AIRCRAFT_NAME|PROPERTY_UNIT] => PROPERTY_VALUE

    asort ($arr, SORT_NUMERIC);  // sort with ascending property-values and preserving keys

    foreach ($arr as $key => $val)
    {
      $arr1 = explode ('|', $key);
      $s .= '<tr>';
      $s .= '<td>' . $arr1[0] . '</td>';
      $s .= '<td>' . $val . '</td>';
      $s .= '<td>' . $arr1[1] . '</td>';
      $s .= '</tr>';
    }

    $s .= '</table>';

    return $s;
  }


  function getOtherAircraftInfos ($arrII, $sCurrentAircraftName)
  {
    $s = '';
    $arr = array ();

    // $arrII (as given as normal parameter) is a copy of $arrInfoIni.
    // Therefor we can delete parts of the array without any harm.

    // clean unnecessary sub-arrays in $arrInfoIni
    unset ($arrII['Calculation']);
    unset ($arrII['Defaults']);

    // fill new array only with necessary data
    foreach ($arrII as $key => $val)
    {
      $arr[$key] = $val['common-name'];
    }

    // sort values of my names
    asort ($arr);

    // fill html select-list
    foreach ($arr as $key => $val)
    {
      $s .= '                  <option value="' . $key . '"' . ($key == $sCurrentAircraftName ? ' selected' : '') . '>' . $val . '</option>' . CR;
    }

    return $s;
  }

?>
